/* 
 * File:   touch_master.h
 * Author: Tahmid
 *
 * Modified by: Carlos Contreras, Kevin Chaudhari
 */

#ifndef TOUCH_MASTER_H
#define	TOUCH_MASTER_H

#include <GenericTypeDefs.h>
#include "plib.h"

#define OVERRIDE_ADC
/* If OVERRIDE_ADC is defined, override external ADC configuration
 * and reconfigure ADC with a default setting.
 * If not, externally have ADC configurations set. Make sure you use
 * a very high sampling time and very slow ADC clock to deal with
 * touch screen impedance charging the S&H cap.
 */

typedef struct ts_struct {
    INT16 X;   // x-coordinate of touch point
    INT16 Y;   // y-coordinate of touch point
    UINT8 Z;    // z = 1 if touch registered, z = 0 otherwise
    UINT8 DONE; // if done = 1, x,y,z hold valid value, otherwise touch
                    // point still being detected
} point_t;

	// X+ = RB10 = pin 21
	// Y+ = RB9 = pin 18
	// X- = RB13 = pin 24 = AN11
	// Y- = RA0 = pin 2 = AN0

#define XP_OUTPUT()		{mPORTBSetPinsDigitalOut(BIT_10);}
#define XP_INPUT()		{mPORTBSetPinsDigitalIn(BIT_10);}
#define XP_LOW()		{mPORTBClearBits(BIT_10);}
#define XP_HIGH()		{mPORTBSetBits(BIT_10);}

#define YP_OUTPUT()		{mPORTBSetPinsDigitalOut(BIT_9);}
#define YP_INPUT()		{mPORTBSetPinsDigitalIn(BIT_9);}
#define YP_LOW()		{mPORTBClearBits(BIT_9);}
#define YP_HIGH()		{mPORTBSetBits(BIT_9);}

#define XM_INPUT()		{mPORTBSetPinsAnalogIn(BIT_13);}
#define XM_OUTPUT()		{mPORTBSetPinsDigitalOut(BIT_13);}
#define XM_HIGH()		{mPORTBSetBits(BIT_13);}
#define XM_LOW()		{mPORTBClearBits(BIT_13);}
#define XM_PD()			{CNPDBSET = BIT_13;}

#define YM_INPUT()		{mPORTASetPinsAnalogIn(BIT_0);}
#define YM_OUTPUT()		{mPORTASetPinsDigitalOut(BIT_0);}
#define YM_HIGH()		{mPORTASetBits(BIT_0);}
#define YM_LOW()		{mPORTAClearBits(BIT_0);}
#define YM_PD()			{CNPDASET = BIT_0;}



#define STATE_DRIVE_READ_X      1
#define STATE_ACQ_X             2
#define STATE_DRIVE_READ_Y      3
#define STATE_ACQ_Y             4

#define TOUCH_THRESHOLD         30

volatile point_t touch_point;
volatile UINT32 TOUCH_STATE;

/**
 * Initialize the touch screen library.
 * If the define OVERRIDE_ADC is defined, then this
 * function configures the ADC as well. If it is not
 * defined, then the ADC must be externally configured.
 *
 */
void touch_init(void);

/**
 * Get the touch screen value in a struct named
 * touch_point. This function contains a state
 * machine with four states and each state should be
 * cycle a few milliseconds apart. 10ms-20ms seems to
 * work fairly well.
 * \n
 * Every time this function is called, check the
 * touch_point struct's DONE field. If it is set to
 * 1, then the other fields in the struct contain valid
 * information. If the Z field is set to 1, that means
 * that a touch was registered and the co-ordinates
 * of the touch have been recorded in the X and Y fields.
 * If Z = 0, X and Y have invalid information that don't
 * mean anything and should be ignored.
 * \n
 * @param None
 */
void touch_getPoint(void);


/**
 * Reset touch parameters
 *
 * \param none
 */
void touch_reset(void);

#endif	/* TOUCH_MASTER_H */

